package SDL::Build;

use strict;
use base 'Module::Build';

use Data::Dumper;
use File::Spec::Functions;

# Module::Build doesn't seem to have a way to use separate flags for separate
# XS files, so here's the override that makes separate files build correctly:
sub process_xs
{
	my ($self, $file) = @_;

	my $properties                   = $self->{properties};
	my $file_args                    = $self->notes( 'file_flags' )->{$file};
	my @old_values                   = @$properties{ keys %$file_args };
	@$properties{ keys %$file_args } = values %$file_args;

	$self->SUPER::process_xs( $file );
	@$properties{ keys %$file_args } = @old_values;
}

# every platform has slightly different library and header paths
sub get_arch
{
	my ($self, $os)   = @_;
	my $modpath       = catfile(
		'SDL', 'Build', ucfirst( $os ) . '.pm' );
	my $module        = 'SDL::Build::' . ucfirst( $os );

	eval { require $modpath } or die "No module for $os platform\n";

	return $module;
}

# subsystem to build
#	file
#		location of source file => location of build file to get name right
#	libraries
#		name of shared library (soname)
#			preprocessor definition
#			name of header file
sub subsystems
{
	my $self = shift;

	return
	{
		SDL => {
			file      =>
			{
				from  => 'src/SDL.xs',
				to    => 'src/SDL_perl.xs',
			},
			libraries => [qw( SDL SDL_image SDL_mixer SDL_net SDL_ttf SDL_gfx
			                  SDL_console png jpeg smpeg )],
		},
		OpenGL =>
		{
			file      => {
				from => 'src/OpenGL.xs',
				to   => 'src/SDL/OpenGL.xs', 
			},
			libraries => [qw( GL GLU )],
		},
		SFont =>
		{
			file => {
				from => 'src/SFont.xs',
				to   => 'src/SDL/SFont.xs',
			},
			libraries => [qw( SDL SDL_image )],
		},
	};
}

# subsystem name
#	defined symbol in the XS files
#	header location
sub libraries
{
	my $self = shift;

	return
	{
		SDL         => {
			define => 'HAVE_SDL',
			header => 'SDL.h',
		},
		SDL_image   => {
			define => 'HAVE_SDL_IMAGE',
			header => 'SDL_image.h'
		},
		SDL_mixer   => {
			define => 'HAVE_SDL_MIXER',   
			header => 'SDL_mixer.h'
		},
		SDL_net     => {
			define => 'HAVE_SDL_NET',     
			header => 'SDL_net.h'
		},
		SDL_ttf     => {
			define => 'HAVE_SDL_TTF',     
			header => 'SDL_ttf.h'
		},
		SDL_gfx     => {
			define => 'HAVE_SDL_GFX',     
			header => 'SDL_gfxPrimitives.h'
		},
		SDL_console => {
			define => 'HAVE_SDL_CONSOLE', 
			header => 'CON_console.h'
		},
		png         => {
			define    => 'HAVE_PNG',
			header    => 'png.h',
		},
		jpeg        => {	
			define    => 'HAVE_JPEG',
			header    => 'jpeglib.h',
		},
		smpeg       => {
			define    => 'HAVE_SMPEG',
			header    => catfile(qw( smpeg smpeg.h )),
		},
		GL          => {
			define => 'HAVE_GL',  
			header => 'gl.h',
		},
		GLU         => {
			define => 'HAVE_GLU', 
			header => 'glu.h',
		},
	};
}

# the user may have passed extra configuration information on the command line
sub command_line_paths
{
	my $self = shift;

	my %paths;

	for my $argument ( @_ )
	{
		next unless $argument   =~ /--with-(\w+)=(.*)$/;
		my ($subsystem, $paths) = ($1, $2);

		my %extra_paths;

		for my $path ( split(/,/, $paths) )
		{
			for my $suff ('', uc( $subsystem ), lc( $subsystem ))
			{
				$extra_paths{ catdir( $path, 'include', $suff ? $suff : () ) }
					        = catdir( $path, 'lib',     $suff ? $suff : () );
			}
		}

		$paths{ $subsystem } = \%extra_paths;
	}

	return %paths;
}

# which headers are installed?
sub find_subsystems
{
	my ($self, $subsystems, $libraries) = @_;
	my %includes_libs                   = $self->fetch_includes();
	my %extra_paths                     = $self->command_line_paths( @ARGV );

	my (%enabled, @disabled);

	while ( my ($name, $subsystem)      = each %$subsystems )
	{
		for my $library (@{ $subsystem->{libraries} })
		{
			my $lib = $libraries->{$library}
				or die "Unknown library '$library' for '$name'\n";

			my $search_headers   = exists $extra_paths{ lc( $name ) } ?
					$extra_paths{ lc( $subsystem ) } :
					\%includes_libs;

			my ($inc_dir, $link_dir) = 
				$self->find_header( $lib->{header}, $search_headers );

			$enabled{$name}{ $library } = $inc_dir ? [ $inc_dir, $link_dir ]
				: 0;
		}

		push @disabled, $name
			unless grep { $enabled{ $name }{ $_ } } keys %{ $enabled{ $name } };
	}

	for my $disabled (@disabled)
	{
		warn "Can't find $disabled headers; skipping.\n" .
			"Ensure that you have its headers installed, then try running:\n" .
			"\t$0 --with-" . lc($disabled) . "=/path/to/directory\n\n";
		delete $subsystems->{ $disabled };
	}

	return \%enabled;
}

# set the define flags for the libraries we have
sub build_defines
{
	my ($self, $libraries, $build_systems) = @_;

	my %defines;

	while (my ($subsystem, $buildable) = each %$build_systems)
	{
		for my $build (grep { $buildable->{ $_ } } keys %$buildable)
		{
			push @{ $defines{ $subsystem } }, "-D$libraries->{$build}{define}";
		}
	}

	return \%defines;
}

# set the include paths for the libraries we have
sub build_includes
{
	my ($self, $libraries, $build_systems) = @_;

	my %includes;

	while (my ($subsystem, $buildable) = each %$build_systems)
	{
		my %sub_include;
		for my $build (grep { $buildable->{ $_ } } keys %$buildable)
		{
			$sub_include{ $buildable->{ $build }[0] }++;
		}

		$includes{ $subsystem } = [ map { "-I$_" } keys %sub_include ];
	}

	return \%includes;
}

# set the linker paths and flags for the libraries we have
sub build_links
{
	my ($self, $libraries, $build_systems) = @_;

	my %links;

	while (my ($subsystem, $buildable)     = each %$build_systems)
	{
		my %sub_links;
		for my $build (grep { $buildable->{ $_ } } keys %$buildable)
		{
			$sub_links{ $buildable->{ $build }[1] }++;
			push @{ $links{ $subsystem }{libs} }, "-l$build";
		}

		$links{ $subsystem }{paths} = [ map { "-L$_" } keys %sub_links ];
	}

	return \%links;
}

# save this all in a format process_xs() can understand
sub set_flags
{
	my ($self, $subsystems, $build, $defines, $includes, $links) = @_;

	my %file_flags;

	while (my ($subsystem, $buildable) = each %$build)
	{
		next unless exists $subsystems->{$subsystem};
		my $sub_file     = $subsystems->{$subsystem}{file}{to};
		my $sub_from     = $subsystems->{$subsystem}{file}{from};

		$self->copy_xs_file( $subsystem, $sub_from, $sub_file );

		my $sub_includes = join(' ', @{ $includes->{$subsystem} } );

		$file_flags{ $sub_file } = 
		{
			extra_compiler_flags => join(' ', 
				@{ $includes->{$subsystem} },
				$self->compile_flags(),
				@{ $defines->{$subsystem} },
				'-fPIC',
			),
			extra_linker_flags => join(' ', 
				@{ $links->{$subsystem}{paths} },
				$self->link_flags(),
				@{ $links->{$subsystem}{libs} },
			),
		},
	}

	$self->notes( 'file_flags' => \%file_flags );
}

# look for a header somewhere on the system
sub find_header
{
	my ($self, $header, $includes) = @_;

	for my $inc_dir (keys %$includes)
	{
		next unless -e catfile( $inc_dir, $header );
		return ($inc_dir, $includes->{$inc_dir});
	}
}

sub compile_flags
{
	my $self  = shift;
	my $flags = `sdl-config --cflags`;

	die "sdl-config does not appear to be in your path.\n" .
		"Please check your config and try again.\n" if $? >> 8;

	chomp  $flags;
	return $flags;
}

sub link_flags
{
	my $self  = shift;
	my $flags = `sdl-config --libs`;

	die "sdl-config does not appear to be in your path.\n" .
		"Please check your config and try again.\n" if $? >> 8;

	chomp  $flags;
	return $flags;
}

sub opengl_headers
{
	return map { uc( $_ ) => catfile( 'GL', $_ . '.h' ) } qw( gl glu );
}

sub copy_xs_file
{
	my ($self, $subsystem, $from, $to) = @_;

	if (my $method  = $self->can( lc( $subsystem ) . '_headers' ))
	{
		my %headers = $self->$method();
		open( my $in, '<', $from ) or die "Cannot read '$from': $!\n";
		open( my $out, '>', $to  ) or die "Cannot write '$to': $!\n";

		my $done;

		while ( <$in> )
		{
			 $_ = $self->replace_header( \%headers, $1, $2 )
			 	if m[(/\* (\w+) headers here \*/)];

			print $out $_;
			last unless keys %headers;
		}

		print $out $_ while <$in>;

		my $locations     = $self->{properties}{xs_files};
		$locations->{$to} = $to;
		delete $locations->{$from};
	}
}

sub replace_header
{
	my ($self, $headers, $line, $type ) = @_;

	return $line unless exists $headers->{ $type };
	return "#include <" . delete( $headers->{ $type } ) . ">\n";
}

sub write_sdl_config
{
	my ($self, $config) = @_;
	my $path            = catfile(qw( lib SDL Config.pm ));
	my $dd              = Data::Dumper->new( [ $config ], [ 'sdl_config' ] );
	my $hash            = $dd->Dump();
	(my $text           = <<'	END_HEADER' . $hash . <<'	END_FOOTER');
	package SDL::Config;

	my $sdl_config; 
	END_HEADER

	sub has
	{
		my ($class, $define) = @_;

		while (my ($name, $subsystems) =  each %$sdl_config)
		{
			next unless exists $subsystems->{ $define };
			return 1    if     $subsystems->{ $define };
		}

		return;
	}

	1;
	END_FOOTER

	$text =~ s/^\t//gm;

	open my $file, '>', $path or die "Cannot write to '$path': $!\n";
	print $file $text;
}

1;

__END__

=head1 NAME

SDL::Build - build process for SDL_perl

=head1 SYNOPSIS

See F<Build.PL> in the SDL_perl distribution for the example usage.

=head1 DESCRIPTION

This module governs the build process for SDL_perl.  Specifically, it allows
building multiple XS files across several platforms.  This is tricky because
different platforms have different ways of referring to libraries and headers
and different requirements for auxiliary libraries.

There are platform-specific subclasses to encapsulate this information; see
C<SDL::Build::Linux>, for example.  You'll likely use one of these by default.
Each subclass can override any of the methods here to provide platform-specific
behavior.

=head1 METHODS

=over 4

=item C<get_arch( $archname )>

Loads the appropriate platform-specific module for the build platform.  This
uses C<$^O> instead of C<Module::Build::os_type()> for more fine-grained
control; FreeBSD differs from Linux in linking in its threading library, for
example.

=item C<process_xs( $xs_file )>

Overrides the parent method in C<Module::Build> to add file-specific flags
before building each XS file.  This is important to add specific linker and
header include flags for different files.

=item C<subsystems()>

Returns a hash of hashes keyed on the name of various subsystems potentially to
build.  Each subhash contains a C<file> key, with a hash containing C<from> and
C<to> locations of where to copy the XS files for building, and a C<libraries>
key with an array of libraries to link against for building this subsystem.

=item C<libraries()>

Returns a hash of hashes keyed on library name (as found in the data returned
from C<subsystems()>) where each library has a C<define> symbol which will be
true if the library exists on the system and a C<header> which identifies the
header that must be present to link against the library.

=item C<command_line_paths( @ARGV )>

Processes any extra paths given on the command line when invoking F<Build.PL>
to point to specific include directories for headers.  For example, to use the
C<SDL_mixer> header found in F</usr/local/include/SDL/>, use:

	$ perl Build.PL --with-SDL_mixer=/usr/local/include/SDL/

=item C<find_subsystems( $subsystems, $libraries )>

Looks through the platform's defined include paths and any paths given on the
command line for the header for each subsystem's library.  If found, it will
enable that library in the build.  Otherwise, it will give a warning message
about disabling that library, suggesting how to fix it.

=item C<build_defines( $libraries, $buildable_subsystems )>

Returns a hash reference of the appropriate defines to set for building each
subsystem, based on the available subsystem libraries.

=item C<build_includes( $libraries, $buildable_subsystems )>

Returns a hash reference of the appropriate header include paths to use for
building each subsystem, based on the available subsystem libraries.

=item C<build_links( $libraries, $buildable_subsystems )>

Returns a hash reference of the appropriate library paths to use for linking
each subsystem, based on the available subsystem libraries.

=item C<set_flags( ... )>

Sets all of the gathered data so that C<process_xs()> can retrieve it
appropriately.  Hopefully you don't have to edit this one, though if it
generates incorrect compiler commands, you will.

=item C<find_header( $header, $include_directories )>

Searches the directories given in the C<$include_directories> hashref for the
given C<$header>, returning the directory of the header and the directory of
the shared library to link against, if found.

=item C<compile_flgas()>

Looks for the compiler flags necessary to compile SDL on this platform; this
uses C<sdl-config>, which you ought to have on your system.

=item C<link_flags()>

Looks for the compiler flags necessary to link against SDL on this platform;
this uses C<sdl-config>, which you ought to have on your system.

=item C<opengl_headers()>

Returns a list of OpenGL subsystem names and their header files.  By default,
the C<GL> library uses a header named F<gl.h> and the C<GLU> library uses a
header named F<glu.h>.  This may be very different on your system.

=item C<copy_xs_file( $subsystem, $from_path, $to_path )>

Copies the file at C<$from_path> to C<$to_path>, rewriting its headers if
necessary; this calls a method named C<${subsystem}_headers> (that is,
C<opengl_headers()> for the OpenGL subsystem) to find the headers to rewrite.
This I<may> be a mistake for OpenGL headers; it's not yet clear what the right
solution is.

=item C<replace_header( $headers, $line, $type )>

Helper method for C<copy_xs_file()>.

=item C<write_sdl_config( $config )>

Writes the C<SDL::Config> module based on the configured and built subsystems.
This makes it possible to upgrade SDL_perl based on an installed configuration
and to test the enabled features in the tests or in real programs.

=back

=head1 AUTHOR

chromatic, E<lt>chromatic at wgz dot orgE<gt>.  Please send all questions,
patches, and bugs to E<lt>sdl-devel at perl dot orgE<gt>.

=head1 BUGS

No known bugs.
